@@ -1,7 +1,7 @@
 /*
  * gloc.h - Public API for Scheme glocs
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -36,17 +36,7 @@
 #ifndef GAUCHE_GLOC_H
 #define GAUCHE_GLOC_H
 
-struct ScmGlocRec {
-    SCM_HEADER;
-    ScmSymbol *name;
-    ScmModule *module;
-    ScmObj value;               /* The actual value.  Have to be accessed
-                                   via SCM_GLOC_{GET|SET} macros. */
-    char hidden;                /* TRUE if this is a "negative binding",
-                                   see below. */
-    ScmObj (*getter)(ScmGloc *);         /* see 'hooks' below */
-    ScmObj (*setter)(ScmGloc *, ScmObj); /* see 'hooks' below */
-};
+/* Definition of ScmGlocRec is in gauche/priv/glocP.h */
 
 /* About negative binding:
  *
@@ -98,28 +88,16 @@ SCM_CLASS_DECL(Scm_GlocClass);
 #define SCM_GLOC(obj)            ((ScmGloc*)(obj))
 #define SCM_GLOCP(obj)           SCM_XTYPEP(obj, SCM_CLASS_GLOC)
 
-#define SCM_GLOC_GET(gloc) \
-    ((gloc)->getter? (gloc)->getter(gloc) : (gloc)->value)
-#define SCM_GLOC_SET(gloc, val) \
-    ((gloc)->setter? (gloc)->setter((gloc), (val)) : ((gloc)->value = (val)))
+/* Scm_GlocGetValue can raise 'reference to a phantom gloc' error if
+   the given gloc is a phantom binding.  Use Scm_GlocPhantomBindingP
+   to exclude such case. */
 
-#define SCM_GLOC_PHANTOM_BINDING_P(gloc) SCM_UNBOUNDP((gloc)->value)
+SCM_EXTERN ScmObj Scm_GlocGetValue(ScmGloc *g);
+SCM_EXTERN void   Scm_GlocSetValue(ScmGloc *g, ScmObj val);
 
+SCM_EXTERN int    Scm_GlocPhantomBindingP(ScmGloc *g);
 SCM_EXTERN int    Scm_GlocConstP(ScmGloc *g);
 SCM_EXTERN int    Scm_GlocInlinableP(ScmGloc *g);
-
-/* INTERNAL */
-
-SCM_EXTERN ScmObj Scm_MakeGloc(ScmSymbol *sym, ScmModule *module);
-/* flags may be 0, SCM_BINDING_CONST, or SCM_BINDING_INLINABLE. */
-SCM_EXTERN void   Scm_GlocMark(ScmGloc *g, int flags);
-SCM_EXTERN ScmObj Scm_GlocConstSetter(ScmGloc *g, ScmObj val);
-SCM_EXTERN ScmObj Scm_GlocInlinableSetter(ScmGloc *g, ScmObj val);
-
-/* For ABI compatibility.  Should be gone at api1.0. */
-#define SCM_GLOC_CONST_P(gloc)  ((gloc)->setter == Scm_GlocConstSetter)
-SCM_EXTERN ScmObj Scm_GlocMarkConst(ScmGloc *g);
-SCM_EXTERN ScmObj Scm_GlocUnmarkConst(ScmGloc *g);
+SCM_EXTERN int    Scm_GlocSupersedableP(ScmGloc *g, u_long flags, ScmObj val);
 
 #endif /* GAUCHE_GLOC_H */
-
