@@ -1,7 +1,7 @@
 /*
  * load.h - Public API for loading files
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -40,14 +40,10 @@
  * Loading Scheme files
  */
 
-/* Flags for Scm_VMLoad (V), Scm_Load (L), Scm_Require (R), Scm_VMLoadPort (P),
- * and Scm_FindFile (F).
- * NB: We intended to use 1L<<3 for MAIN_SCRIPT flag but it turned out
- * unnecessary.
- */
+/* Flags for Scm_VMLoad (V), Scm_Load (L), Scm_Require (R) */
 typedef enum {
     SCM_LOAD_QUIET_NOFILE = (1L<<0),
-    /* [L,V,F] do not signal an error if the file does not exist;
+    /* [L,V] do not signal an error if the file does not exist;
        just return #f. */
 
     SCM_LOAD_IGNORE_CODING = (1L<<1),
@@ -56,10 +52,9 @@ typedef enum {
     SCM_LOAD_PROPAGATE_ERROR = (1L<<2),
     /* [L,R] do not capture an error; let the caller handle it.  */
 
-    SCM_LOAD_SEARCH_ARCHIVE = (1L<<4)
-    /* [F] Search a file to load from archive file, using the hook of
-       Scm_FindFile.  This is mainly for internal use---Scm_VMLoad etc calls
-       Scm_FindFile with this flag on. */
+    SCM_LOAD_MAIN_SCRIPT = (1L<<4)
+    /* [L,V] loading main script.  We set full path of the loaded script
+       to `scrilt-file`.  */
 } ScmLoadFlags;
 
 /* A structure to obtain a detailed result of loading. */
@@ -68,8 +63,6 @@ typedef struct ScmLoadPacketRec {
     int    loaded;    /* OUT: TRUE iff file is successfully loaded.  */
 } ScmLoadPacket;
 
-SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
-                                     ScmObj env, int flags);
 SCM_EXTERN ScmObj Scm_VMLoad(ScmString *file, ScmObj paths, ScmObj env,
                              int flags);
 
@@ -85,7 +78,6 @@ SCM_EXTERN int Scm_LoadFromCString(const char *program, u_long flags,
 SCM_EXTERN ScmObj Scm_CurrentLoadHistory(void);
 SCM_EXTERN ScmObj Scm_CurrentLoadNext(void);
 SCM_EXTERN ScmObj Scm_CurrentLoadPort(void);
-SCM_EXTERN ScmObj Scm_LoadMainScript(void);
 
 /*=================================================================
  * Load path management
@@ -109,6 +101,10 @@ SCM_CLASS_DECL(Scm_DLObjClass);
 SCM_EXTERN ScmObj Scm_DynLoad(ScmString *path, ScmObj initfn, u_long flags);
 
 SCM_EXTERN ScmObj Scm_DLObjs(void);
+SCM_EXTERN ScmObj Scm_DLOGetEntryAddress(ScmDLObj *dlo, ScmString *name);
+
+SCM_EXTERN int Scm_DLPtrP(ScmObj obj);
+SCM_EXTERN ScmObj Scm_DLPtrValue(ScmObj obj);
 
 /*=================================================================
  * Require & Provide
@@ -157,5 +153,11 @@ SCM_EXTERN void   Scm_DefineAutoload(ScmModule *where, ScmObj file_or_module,
                                      ScmObj list);
 SCM_EXTERN ScmObj Scm_ResolveAutoload(ScmAutoload *autoload, int flags);
 
-#endif /* GAUCHE_LOAD_H */
+/* Obsoleted stuff */
+#if GAUCHE_API_VERSION < 98
+SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
+                                     ScmObj env, int flags);
+SCM_EXTERN ScmObj Scm_LoadMainScript(void);
+#endif /*GAUCHE_API_VERSION < 98*/
 
+#endif /* GAUCHE_LOAD_H */
