@@ -1,7 +1,7 @@
 /*
  * module.h - Modules
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -69,6 +69,9 @@ struct ScmModuleRec {
     ScmObj info;                /* alist of metainfo; e.g.
                                    (source-info . <string>) */
     int    sealed;              /* if true, no modification is allowed */
+    int    placeholding;        /* if true, this module is created just for
+                                   hygienic identifiers, and the module body
+                                   isn't loaded. */
 };
 
 #define SCM_MODULE(obj)       ((ScmModule*)(obj))
@@ -85,10 +88,19 @@ enum {
     SCM_BINDING_STAY_IN_MODULE = (1L<<0), /*(F,R) do not search parent/imported*/
     SCM_BINDING_CONST = (1L<<1),          /*(M) constant binding */
     SCM_BINDING_INLINABLE = (1L<<2),      /*(M) inlinable binding */
-    SCM_BINDING_EXTERNAL = (1L<<3)        /*(F) only search externally visible
+    SCM_BINDING_EXTERNAL = (1L<<3),       /*(F) only search externally visible
                                             bindings, as if we're importing
                                             the module.  Currently used to
                                             create alias binding. */
+    SCM_BINDING_DUMMY = (1L<<4)           /*(M) inline or constant binding,
+                                            inserted only during compilation.
+                                            this is a dummy binding only to be
+                                            recognized by the compiler to expand
+                                            the following expressions. The
+                                            binding is ot be overwritten by
+                                            the proper one when the compiled
+                                            module is loaded, so we won't warn
+                                            the overwriting. */
 };
 
 SCM_EXTERN ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
@@ -132,9 +144,12 @@ SCM_EXTERN void   Scm_ModuleSeal(ScmModule *mod);
    module doesn't exist.  This change should be transparent as far
    as the caller's using Gauche's definition of TRUE. */
 enum {
-    SCM_FIND_MODULE_CREATE = 1, /* Create if there's no named module */
-    SCM_FIND_MODULE_QUIET  = 2  /* Do not signal an error if there's no
-                                   named module, but return NULL instead. */
+    SCM_FIND_MODULE_CREATE = (1L<<0), /* Create if there's no named module */
+    SCM_FIND_MODULE_QUIET  = (1L<<1), /* Do not signal an error if there's no
+                                         named module, but return NULL
+                                         instead. */
+    SCM_FIND_MODULE_PLACEHOLDING = (1L<<2) /* If module is newly created,
+                                              mark it as placeholding. */
 };
 
 #define SCM_FIND_MODULE(name, flags) \
@@ -163,4 +178,3 @@ SCM_EXTERN ScmModule *Scm_CurrentModule(void);
 
 
 #endif /*GAUCHE_MODULE_H*/
-
