@@ -1,7 +1,7 @@
 /*
  * number.h - Public API for Scheme numbers
  *
- *   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -194,6 +194,9 @@ SCM_EXTERN int    Scm_ClampMode(ScmObj clamp);
 
 SCM_EXTERN ScmObj Scm_MakeInteger(long i);
 SCM_EXTERN ScmObj Scm_MakeIntegerU(u_long i);
+SCM_EXTERN ScmObj Scm_MakeIntegerFromUIArray(int sign,
+                                             const u_long *values,
+                                             int size);
 
 SCM_EXTERN long   Scm_GetIntegerClamp(ScmObj obj, int clamp, int *oor);
 SCM_EXTERN u_long Scm_GetIntegerUClamp(ScmObj obj, int clamp, int *oor);
@@ -258,11 +261,11 @@ SCM_EXTERN ScmObj Scm_MakeCompnum(double real, double imag);
 SCM_EXTERN ScmObj Scm_MakeComplex(double real, double imag);
 SCM_EXTERN ScmObj Scm_MakeComplexPolar(double magnitude, double angle);
 SCM_EXTERN ScmHalfComplex Scm_GetHalfComplex(ScmObj obj);
-SCM_EXTERN complex float  Scm_GetFloatComplex(ScmObj obj);
-SCM_EXTERN complex double Scm_GetDoubleComplex(ScmObj obj);
+SCM_EXTERN ScmFloatComplex  Scm_GetFloatComplex(ScmObj obj);
+SCM_EXTERN ScmDoubleComplex Scm_GetDoubleComplex(ScmObj obj);
 SCM_EXTERN ScmObj Scm_HalfComplexToComplex(const ScmHalfComplex c);
-SCM_EXTERN ScmObj Scm_FloatComplexToComplex(const complex float c);
-SCM_EXTERN ScmObj Scm_DoubleComplexToComplex(const complex double c);
+SCM_EXTERN ScmObj Scm_FloatComplexToComplex(const ScmFloatComplex c);
+SCM_EXTERN ScmObj Scm_DoubleComplexToComplex(const ScmDoubleComplex c);
 
 SCM_EXTERN int    Scm_IntegerP(ScmObj obj);
 SCM_EXTERN int    Scm_OddP(ScmObj obj);
@@ -355,8 +358,14 @@ enum ScmNumberFormatFlags {
                                               decimal notation, then round.
                                               [S] ignored.
                                             */
-    SCM_NUMBER_FORMAT_STRICT_R7RS = (1L<<4) /*[N] ignored.
+    SCM_NUMBER_FORMAT_STRICT_R7RS = (1L<<4),/*[N] ignored.
                                               [S] reject syntax outside of R7RS*/
+    SCM_NUMBER_FORMAT_EXACT = (1L<<5),    /* [N] ignored.
+                                             [S] treat as if #e is present if
+                                             no exactness prefix is given */
+    SCM_NUMBER_FORMAT_INEXACT = (1L<<6)   /* [N] ignored.
+                                             [S] treat as if #i is present if
+                                             no exactness prefix is given */
 };
 
 typedef struct ScmNumberFormatRec {
@@ -376,11 +385,6 @@ SCM_EXTERN size_t Scm_PrintDouble(ScmPort *port, double d, ScmNumberFormat *f);
 SCM_EXTERN ScmObj Scm_NumberToString(ScmObj num, int radix, u_long flags);
 SCM_EXTERN ScmObj Scm_StringToNumber(ScmString *str, int radix, u_long flags);
 
-/* This is here, for we need to check double endianness on ARM. */
-SCM_EXTERN ScmObj Scm_NativeEndian(void);
-SCM_EXTERN ScmObj Scm_DefaultEndian(void);
-SCM_EXTERN void   Scm_SetDefaultEndian(ScmObj endian);
-
 /* The Scm_VM* version leaves unboxed flonum (FLONUM_REG) in VM's VAL0
    register.   They can only be called "on VM", that is, when its return
    value is used immediately by VM. */
@@ -400,4 +404,3 @@ SCM_EXTERN ScmObj Scm_VMExpt(ScmObj x, ScmObj y);
 SCM_EXTERN ScmObj Scm_VMRound(ScmObj num, int mode);
 
 #endif /* GAUCHE_NUMBER_H */
-
