@@ -1,7 +1,7 @@
 /*
  * string.h - Public API for Scheme strings
  *
- *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -70,10 +70,10 @@
    the backward compatibility.
 */
 /* The 'index' slot may contain an index vector to realize O(1) random-access
- * of the string.  Building index costs time and space, so it is only 
+ * of the string.  Building index costs time and space, so it is only
  * constructed when explicitly asked.  Srfi-135 (Immutable Texts) is
  * really an immutable string with an indexed body.
- * The user should treat index field as a opaque pointer.  
+ * The user should treat index field as a opaque pointer.
  * See priv/stringP.h for the details.
  */
 
@@ -199,7 +199,7 @@ SCM_EXTERN ScmObj  Scm_MakeString(const char *str,
                                   ScmSmallInt size, ScmSmallInt len,
                                   u_long flags);
 SCM_EXTERN ScmObj  Scm_MakeFillString(ScmSmallInt len, ScmChar fill);
-SCM_EXTERN ScmObj  Scm_CopyStringWithFlags(ScmString *str, 
+SCM_EXTERN ScmObj  Scm_CopyStringWithFlags(ScmString *str,
                                            u_long flags, u_long mask);
 
 #define SCM_MAKE_STR(cstr) \
@@ -310,7 +310,7 @@ enum {
     SCM_STRING_SCAN_BEFORE2,    /* return substr of s1 before s2 and rest */
     SCM_STRING_SCAN_AFTER2,     /* return substr of s1 up to s2 and rest */
     SCM_STRING_SCAN_BOTH,       /* return substr of s1 before and after s2 */
-    SCM_STRING_SCAN_CURSOR,     /* return <string-cursor> 
+    SCM_STRING_SCAN_CURSOR,     /* return <string-cursor>
                                    TRANSIENT: This may be more reasonable
                                    to be after INDEX, but we have it here
                                    until 1.0 to keep ABI compatibility */
@@ -379,8 +379,8 @@ SCM_EXTERN ScmSmallInt Scm_DStringSize(ScmDString *dstr);
 SCM_EXTERN ScmObj      Scm_DStringGet(ScmDString *dstr, u_long flags);
 SCM_EXTERN const char *Scm_DStringGetz(ScmDString *dstr);
 SCM_EXTERN void        Scm_DStringWeld(ScmDString *dstr);
-SCM_EXTERN const char *Scm_DStringPeek(ScmDString *dstr, 
-                                       ScmSmallInt *size, 
+SCM_EXTERN const char *Scm_DStringPeek(ScmDString *dstr,
+                                       ScmSmallInt *size,
                                        ScmSmallInt *len);
 SCM_EXTERN void        Scm_DStringPutz(ScmDString *dstr, const char *str,
                                        ScmSmallInt siz);
@@ -419,37 +419,4 @@ SCM_EXTERN void Scm__DStringRealloc(ScmDString *dstr, ScmSmallInt min_incr);
  */
 SCM_EXTERN char *Scm_StrdupPartial(const char *src, size_t size);
 
-#if GAUCHE_STRING_POINTER
-/*
- * String pointers (OBSOLETED)
- */
-
-/* Efficient way to access string from Scheme */
-typedef struct ScmStringPointerRec {
-    SCM_HEADER;
-    ScmSmallInt length;
-    ScmSmallInt size;
-    const char *start;
-    ScmSmallInt index;
-    const char *current;
-} ScmStringPointer;
-
-SCM_CLASS_DECL(Scm_StringPointerClass);
-#define SCM_CLASS_STRING_POINTER  (&Scm_StringPointerClass)
-#define SCM_STRING_POINTERP(obj)  SCM_XTYPEP(obj, SCM_CLASS_STRING_POINTER)
-#define SCM_STRING_POINTER(obj)   ((ScmStringPointer*)obj)
-
-SCM_EXTERN ScmObj Scm_MakeStringPointer(ScmString *src, ScmSmallInt index,
-                                        ScmSmallInt start, ScmSmallInt end);
-SCM_EXTERN ScmObj Scm_StringPointerRef(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerNext(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerPrev(ScmStringPointer *sp);
-SCM_EXTERN ScmObj Scm_StringPointerSet(ScmStringPointer *sp,
-                                       ScmSmallInt index);
-SCM_EXTERN ScmObj Scm_StringPointerSubstring(ScmStringPointer *sp, int beforep);
-SCM_EXTERN ScmObj Scm_StringPointerCopy(ScmStringPointer *sp);
-SCM_EXTERN void   Scm_StringPointerDump(ScmStringPointer *sp);
-
-#endif /*GAUCHE_STRING_POINTER*/
-
 #endif /* GAUCHE_STRING_H */
