@@ -1,7 +1,7 @@
 ;;;
 ;;;  data.random - Random data genarators
 ;;;
-;;;   Copyright (c) 2013-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,18 +32,18 @@
 ;;;
 
 (define-module data.random
-  (use srfi-1)
-  (use srfi-14)
+  (use scheme.list)
+  (use scheme.charset)
+  (use srfi-27)
   (use srfi-42)
   (use util.match)
   (use math.const)
   (use math.mt-random)
   (use gauche.uvector)
   (use gauche.generator)
-  (use gauche.parameter)
   (use gauche.sequence)
-  
-  (export make-random-data-state random-data-seed with-random-data-seed
+
+  (export random-data-random-source random-data-seed with-random-data-seed
 
           integers$ integers-between$ fixnums chars$ samples$ booleans
           int8s uint8s int16s uint16s int32s uint32s int64s uint64s
@@ -58,35 +58,46 @@
           ))
 (select-module data.random)
 
-;; Random state management
-;; Random state is kept in %random-source parameter as (seed . #<mt>)
-;; We use mt-random directly instead of srfi-27, for we need
-;; a portable way to save and restore the random state.
-;; Srfi-27's random state isn't guaranteed to be printable.
-
 ;; API
-(define (make-random-data-state seed)
-  (cons seed (make <mersenne-twister> :seed seed)))
+;;  Use srfi-27's default-random-source as the default, for the compatibility
+;;  to srfi-194.
+;;  In the current implementation, this is <mersenne-twister> instance, and
+;;  unless the user explicitly randomize it, it uses a fixed predetermined
+;;  seed.  It guarantees reproducibility.
+(define random-data-random-source
+  (make-parameter default-random-source))
 
-(define %random-data-state
-  (make-parameter (make-random-data-state 42)))
+;; random-data-seed and with-random-data-seed are intended to reproduce
+;; sequences that has been generated.  If you simply using srfi-27
+;; random-source as a parameter, you can't retrieve its seed value,
+;; so you can't reproduce the sequence unless you remember what seed value
+;; you used to initialize the random-source.
+;; However, the interface is kind of awkward (e.g. you can't swap
+;; two independent random sources) and non-intuitive.   We keep them
+;; only for the backward compatibility.
 
-;; API
+;; Deprecated API
+;; NB: Currently the only srfi-27 random-source is <mersenne-twister>,
+;; but we may add other kind of random-source in future.
 (define random-data-seed
   (getter-with-setter
-   (^[] (car (%random-data-state)))
-   (^[seed] (%random-data-state (make-random-data-state seed)))))
+   (^[] (let1 rs (random-data-random-source)
+          (if (is-a? rs <mersenne-twister>)
+            (mt-random-get-seed rs)
+            (undefined))))
+   (^[seed] (random-data-random-source
+             (make <mersenne-twister> :seed seed)))))
 
-;; API
+;; Deprecated API
 (define (with-random-data-seed seed thunk)
   ;; create st here so that reentering thunk retains the state.
-  (let1 st (make-random-data-state seed)
-    (parameterize ([%random-data-state st])
+  (let1 st (make <mersenne-twister> :seed seed)
+    (parameterize ([random-data-random-source st])
       (thunk))))
 
-(define (%rand-int n) (mt-random-integer (cdr (%random-data-state)) n))
-(define (%rand-real0) (mt-random-real0 (cdr (%random-data-state))))
-(define (%rand-real)  (mt-random-real (cdr (%random-data-state))))
+(define (%rand-int n) (mt-random-integer (random-data-random-source) n))
+(define (%rand-real0) (mt-random-real0 (random-data-random-source)))
+(define (%rand-real)  (mt-random-real (random-data-random-source)))
 
 ;;;
 ;;; Primitive generators
@@ -114,7 +125,7 @@
 (define uint16s (integers$ 65536))
 (define int32s  (integers$ (expt 2 32) (- (expt 2 31))))
 (define uint32s (integers$ (expt 2 32)))
-(define int64s  (integers$ (expt 2 64) (- (expt 2 64))))
+(define int64s  (integers$ (expt 2 64) (- (expt 2 63))))
 (define uint64s (integers$ (expt 2 64)))
 
 ;; API.
@@ -129,7 +140,7 @@
   ;; the delimited ranges of characters in CSET.  For example, if CSET
   ;; is splitted into a ranges ((48 . 57) (65 . 90) (97 . 122)),
   ;; the table has the following mappings:
-  ;;    0 -> 48         (for range 48-57) 
+  ;;    0 -> 48         (for range 48-57)
   ;;   10 -> (- 65 10)  (for range 65-90)
   ;;   36 -> (- 97 36)  (for range 97-122)
   ;; We generate a random integer in [0, 61], that is, the size same as
@@ -324,10 +335,10 @@ plot 'tmp' using (bin($1,binwidth)):(1.0) smooth freq with boxes
       [((or 'assert 'nassert) . _)
        (error "Lookahead/behind assertion can't be used (yet) to \
                generate strings:" (regexp-unparse ast))]
-      [((? integer? n) _ . as) 
+      [((? integer? n) _ . as)
        (let1 nodes (map rec as)
          (^[case-fold] (every (cut <> case-fold) nodes)))]
-      [_ (error "Unsupported regexp to generate strings:" 
+      [_ (error "Unsupported regexp to generate strings:"
                 (regexp-unparse ast))]))
 
   (rec ast))
@@ -389,7 +400,7 @@ plot 'tmp' using (bin($1,binwidth)):(1.0) smooth freq with boxes
   (case-lambda
     [()         (strings-of (default-sizer) (chars$))]
     [(item-gen) (strings-of (default-sizer) item-gen)]
-    [(sizer item-gen) 
+    [(sizer item-gen)
      (%with-sizer sizer
        (^[] (let1 len (sizer)
               (with-output-to-string
@@ -408,7 +419,7 @@ plot 'tmp' using (bin($1,binwidth)):(1.0) smooth freq with boxes
 
 ;; API
 (define (permutations-of seq)
-  (^[] (shuffle seq (cdr (%random-data-state)))))
+  (^[] (shuffle seq (random-data-random-source))))
 
 ;; API
 (define (combinations-of len seq)
@@ -416,4 +427,3 @@ plot 'tmp' using (bin($1,binwidth)):(1.0) smooth freq with boxes
     (^[] (let1 ix (shuffle indices)
            (coerce-to (class-of seq)
                       (list-ec (: i len) (ref seq (vector-ref ix i))))))))
-
