@@ -1,7 +1,7 @@
 ;;;
 ;;; Array operations.  This is superset of SRFI-25.
 ;;;
-;;;  Copyright (c) 2002-2018  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2002-2021  Shiro Kawai  <shiro@acm.org>
 ;;;  Copyright(C) 2004      by Alex Shinn  (foof@synthcode.com)
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
@@ -18,7 +18,7 @@
 ;; map n-dimensional indices to an index of the backing storage.
 
 (define-module gauche.array
-  (use srfi-1)
+  (use scheme.list)
   (use srfi-11)
   (use gauche.uvector)
   (use gauche.collection)
@@ -41,20 +41,32 @@
           make-f16array make-f32array make-f64array
           u8array s8array u16array s16array u32array s32array
           u64array s64array f16array f32array f64array
-          array-concatenate array-transpose array-rotate-90 array-flip array-flip!
-          identity-array array-inverse determinant determinant! array-mul array-expt
-          array-div-left array-div-right array-add-elements array-add-elements!
-          array-sub-elements array-sub-elements! array-mul-elements array-mul-elements!
-          array-div-elements array-div-elements! pretty-print-array
+          array-concatenate array-transpose array-rotate-90
+          array-flip array-flip!
+          identity-array array-inverse determinant determinant!
+          array-mul array-expt
+          array-div-left array-div-right
+          array-add-elements array-add-elements!
+          array-sub-elements array-sub-elements!
+          array-negate-elements array-negate-elements!
+          array-mul-elements array-mul-elements!
+          array-div-elements array-div-elements!
+          array-reciprocate-elements array-reciprocate-elements!
+          pretty-print-array
           ))
 (select-module gauche.array)
 
 (autoload "gauche/matrix"
   array-concatenate array-transpose array-rotate-90 array-flip array-flip!
   identity-array array-inverse determinant determinant! array-mul array-expt
-  array-div-left array-div-right array-add-elements array-add-elements!
-  array-sub-elements array-sub-elements! array-mul-elements array-mul-elements!
-  array-div-elements array-div-elements! pretty-print-array)
+  array-div-left array-div-right
+  array-add-elements array-add-elements!
+  array-sub-elements array-sub-elements!
+  array-negate-elements array-negate-elements!
+  array-mul-elements array-mul-elements!
+  array-div-elements array-div-elements!
+  array-reciprocate-elements array-reciprocate-elements!
+  pretty-print-array)
 
 (define-class <array-meta> (<class>)
   ((backing-storage-creator :init-keyword :backing-storage-creator
@@ -691,4 +703,3 @@
       (^[ind] (add! (array-ref ar ind)))
       (make-vector (array-rank ar)))
     (get)))
-
