@@ -1,7 +1,7 @@
 ;;;
 ;;; configure.scm - configuring Gauche extensions
 ;;;
-;;;   Copyright (c) 2013-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -70,7 +70,6 @@
 ;; TODO: Caching test results
 
 (define-module gauche.configure
-  (use gauche.parameter)
   (use gauche.generator)
   (use gauche.dictionary)
   (use gauche.parseopt)
@@ -94,7 +93,7 @@
           cf-msg-checking cf-msg-result cf-msg-warn cf-msg-error cf-msg-notice
           cf-echo
           cf-make-gpd
-          cf-define cf-subst cf-subst-append cf-subst-prepend
+          cf-define cf-defined? cf-subst cf-subst-append cf-subst-prepend
           cf-arg-var cf-have-subst? cf-ref cf$
           with-cf-subst
           cf-config-headers cf-output cf-output-default cf-show-substs
@@ -105,13 +104,14 @@
           cf-lang-program cf-lang-io-program cf-lang-call
           cf-try-compile cf-try-compile-and-link
 
-          cf-check-header cf-check-headers cf-includes-default
-          cf-check-type cf-check-types
-          cf-check-decl cf-check-decls
-          cf-check-member cf-check-members
-          
-          cf-check-func cf-check-funcs
-          cf-check-lib cf-search-libs
+          cf-check-header cf-header-available? cf-check-headers
+          cf-includes-default
+          cf-check-type cf-type-available? cf-check-types
+          cf-check-decl cf-decl-available? cf-check-decls
+          cf-check-member cf-member-available? cf-check-members
+
+          cf-check-func cf-func-available? cf-check-funcs
+          cf-check-lib cf-lib-available? cf-search-libs
           ))
 (select-module gauche.configure)
 
@@ -165,7 +165,7 @@
 
 ;;;
 ;;; Basic APIs
-;;; 
+;;;
 
 ;; API
 ;; Like AC_MSG_*
@@ -336,23 +336,35 @@
   (cf-subst 'cross_compiling "no")
   (cf-subst 'subdirs "")
 
+  ;; For cross compilation.  These can be overridden by --build, --host and
+  ;; --target.
+  (cf-subst 'build  (gauche-architecture))
+  (cf-subst 'host   (gauche-architecture))
+  (cf-subst 'target (gauche-architecture))
+
   ;; NB: Autoconf uses AC_PROG_CC to set up CC.  However, we need
   ;; to use the same C compiler with which Gauche was compiled, so
-  ;; so we set it as the default.  We also allow env overrides for
-  ;; some common variables.  (In autoconf, AC_PROG_CC issues AC_ARG_VAR
+  ;; we set it as the default.  We also allow env overrides for some
+  ;; common variables.  (In autoconf, AC_PROG_CC issues AC_ARG_VAR
   ;; for them).
-  (cf-subst 'CC       (gauche-config "--cc"))
+  (cf-subst 'CC (gauche-config "--cc"))
+  (cf-subst 'CFLAGS (gauche-config "--default-cflags"))
   (cf-arg-var 'CPP)
-  (cf-arg-var 'CPPFALGS)
+  (cf-arg-var 'CPPFLAGS)
   (cf-arg-var 'CC)
   (cf-arg-var 'CFLAGS)
   (cf-arg-var 'LDFLAGS)
   (cf-arg-var 'LIBS)
-  ;; NB: Autoconf detemines these through tests, but we already
+  ;; NB: Autoconf determines these through tests, but we already
   ;; know them at the time Gauche is configured.
   (cf-subst 'SOEXT  (gauche-config "--so-suffix"))
   (cf-subst 'OBJEXT (gauche-config "--object-suffix"))
   (cf-subst 'EXEEXT (gauche-config "--executable-suffix"))
+
+  ;; '-DUNICODE' if gauche is in utf8 and on windows.
+  (cf-subst 'WINDOWS_UNICODE_FLAG
+            (cond-expand [(and gauche.os.windows gauche.ces.utf8) "-DUNICODE"]
+                         [else ""]))
   )
 
 (define (parse-command-line-arguments)
@@ -403,7 +415,7 @@
           (exit 0)]
          ;; -x-includes
          ;; -x-libraries
-         
+
          ;; Deal with --enable|--disable|--with|--without options.
          ;; Note that parse-options split '=arg' part of --enable-*=arg or
          ;; --with-*=arg; if it is given, it will be in (car rest).
@@ -440,6 +452,10 @@
         [else (print "Invalid argument: " arg)
               (print "Type `./configure --help' for usage.")
               (exit 1)]))
+    ;; build, host and target override
+    (when (cf-have-subst? 'build_alias)  (cf-subst 'build (cf$ 'build_alias)))
+    (when (cf-have-subst? 'host_alias)   (cf-subst 'host (cf$ 'host_alias)))
+    (when (cf-have-subst? 'target_alias) (cf-subst 'target (cf$ 'target_alias)))
     ))
 
 (define (check-directory-names)
@@ -466,6 +482,7 @@
   (cf-subst 'top_builddir (cf$ 'builddir))
   )
 
+;; Process the arg-if-given and arg-if-not-given callbacks of cf-arg-*.
 (define (process-args)
   (dolist [entry (arg-processors)]
     (match entry
@@ -503,7 +520,7 @@
          (cf-msg-error "Unfulfilled dependency of package ~a: \
                         required to be ~s, but only found ~s."
                        (car req) (cadr req) vers)])))
-  
+
   (and-let* ([reqs (~ (current-package)'gpd'require)]
              [ (not (null? reqs)) ])
     (cf-msg-checking "package dependencies")
@@ -601,7 +618,7 @@
   (cf-path-prog 'GAUCHE_CESCONV  "gauche-cesconv")
 
   (cf-subst 'default_prefix (gauche-config "--prefix"))
- 
+
   (cf-subst 'GAUCHE_PKGINCDIR  (gauche-config "--pkgincdir"))
   (cf-subst 'GAUCHE_PKGLIBDIR  (gauche-config "--pkglibdir"))
   (cf-subst 'GAUCHE_PKGARCHDIR (gauche-config "--pkgarchdir"))
@@ -616,12 +633,20 @@
   (assume-type symbol <symbol>)
   (dict-put! (~ (ensure-package)'defs) symbol value))
 
+;; API
+(define (cf-defined? symbol)
+  (assume-type symbol <symbol>)
+  (dict-exists? (~ (ensure-package)'defs) symbol))
+
 ;; API
 ;; Like AC_SUBST, but we require value (instead of implicitly referencing
 ;; a global variable.
-(define (cf-subst symbol value)
+(define (cf-subst symbol :optional (value #f))
   (assume-type symbol <symbol>)
-  (dict-put! (~ (ensure-package)'substs) symbol value))
+  (if value
+    (dict-put! (~ (ensure-package)'substs) symbol value)
+    (unless (cf-have-subst? symbol)
+      (dict-put! (~ (ensure-package)'substs) symbol ""))))
 
 ;; API
 (define (cf-subst-prepend symbol value :optional (delim " ") (default ""))
@@ -651,8 +676,9 @@
 (define (cf-arg-var symbol)
   (assume-type symbol <symbol>)
   (update! (~ (ensure-package)'precious) (cut set-adjoin! <> symbol))
-  (and-let1 v (sys-getenv (x->string symbol))
-    (cf-subst symbol v)))
+  (if-let1 v (sys-getenv (x->string symbol))
+    (cf-subst symbol v)
+    (cf-subst symbol)))
 
 (define (var-precious? symbol)
   (assume-type symbol <symbol>)
@@ -672,7 +698,7 @@
 
 ;; API
 ;; Temporarily replace cf subst value
-(define-syntax with-cf-subst 
+(define-syntax with-cf-subst
   (syntax-rules ()
     [(_ ((var val) ...) body ...)
      (let ([saves (map (cut cf$ <>) '(var ...))])
@@ -726,7 +752,7 @@
   (push! (arg-processors)
          (list feature 'enable help-string
                action-if-given action-if-not-given)))
-  
+
 ;; API
 (define (cf-arg-with package help-string
                      :optional (action-if-given (^[val] #f))
@@ -781,7 +807,14 @@
   (define base-substs (~ pa'substs))
   (define (make-defs)
     (if (null? (~ pa'config.h))
-      (string-join (dict-map (~ pa'defs) (^[k v] #"-D~|k|=~|v|")) " ")
+      (string-join
+       (dict-map (~ pa'defs)
+                 (^[k v]
+                   (let1 vv ($ regexp-replace-all* (x->string v)
+                               #/\\/ "\\\\\\\\"
+                               #/\"/ "\\\\\"")
+                     #"-D~|k|=~|vv|")))
+       " ")
       "-DHAVE_CONFIG_H"))
   (define (make-subst path-prefix)
     (receive (srcdir top_srcdir builddir top_builddir)
@@ -797,7 +830,7 @@
         (^[m]
           (let1 name (string->symbol (m 1))
             (or (dict-get substs name #f)
-                (begin (warn "@~a@ isn't substituted." name)
+                (begin (warn "@~a@ isn't substituted.\n" name)
                        #"@~|name|@")))))))
   ;; We use '/' in the replaced pathname even on Windows; that's what
   ;; autoconf-generated configure does, and it's less likely to confuse
@@ -823,7 +856,7 @@
                     builddir
                     (simplify-path+ (build-path builddir path-prefix)))
                   (simplify-path+ (build-path revpath tbuilddir)))))))
-                  
+
   (define (make-replace-1 output-file)
     (let1 subst (make-subst (sys-dirname (simplify-path+ output-file)))
       (^[line outp]
@@ -845,7 +878,7 @@
     (cf-subst 'prefix (cf$ 'default_prefix)))
   (when (equal? (cf$ 'exec_prefix) "NONE")
     (cf-subst 'exec_prefix "${prefix}"))
-  
+
   (dolist [f files]
     (let1 inf (build-path (cf$'srcdir) #"~|f|.in")
       (unless (file-is-readable? inf)
@@ -864,8 +897,32 @@
       (cf-msg-notice "configure: creating ~a" (car h))
       (file-filter-for-each (make-config.h) :input inf :output (car h)
                             :temporary-file #t :leave-unchanged #t)))
+
+  ;; Record output variables and definitions to config.log
+  (log-output-substs)
+  (log-output-defs)
   )
 
+(define (log-output-substs)
+  (log-format ".")
+  (log-format "## ----------------- ##")
+  (log-format "## Output variables. ##")
+  (log-format "## ----------------- ##")
+  (log-format ".")
+  (dolist [k (sort (hash-table-keys (~ (current-package)'substs)))]
+    (log-format "~a=~a" k (shell-escape-string
+                           (hash-table-get (~ (current-package)'substs) k)))))
+
+(define (log-output-defs)
+  (log-format ".")
+  (log-format "## ------------ ##")
+  (log-format "## Definitions. ##")
+  (log-format "## ------------ ##")
+  (log-format ".")
+  (dolist [k (sort (hash-table-keys (~ (current-package)'defs)))]
+    (log-format "#define ~a ~a" k
+                (or (hash-table-get (~ (current-package)'defs) k) "/**/"))))
+
 ;; API
 ;; Show definitions.
 (define (cf-show-substs :key (formatter (^[k v] (format #t "~16s ~s" k v))))
@@ -1061,8 +1118,9 @@
   (cf-arg-var 'CXX)
   (cf-arg-var 'CXXFLAGS)
   (cf-arg-var 'CCC)
-  (or (cf-ref 'CXX #f)
-      (and-let1 ccc (cf-ref 'CCC #f)
+  (or (not (string-null? (cf-ref 'CXX)))
+      (and-let* ([ccc (cf-ref 'CCC)]
+                 [ (not (string-null? ccc)) ])
         (cf-subst 'CXX ccc)
         #t)
       (cf-check-tool 'CXX compilers :default "g++"))
@@ -1106,7 +1164,8 @@
 ;; If SYM is already cf-subst'd, we don't do anything.
 (define (cf-check-prog sym prog-or-progs
                        :key (value #f) (default #f) (paths #f) (filter #f))
-  (unless (cf-have-subst? sym)
+  (unless (and (cf-have-subst? sym)
+               (not (string-null? (cf-ref sym))))
     (cf-msg-checking "for ~a" prog-or-progs)
     (if-let1 found (check-for-program (listify prog-or-progs)
                                       :paths paths :filter filter)
@@ -1120,7 +1179,8 @@
 ;; cf-path-prog works like AC_PATH_PROG and AC_PATH_PROGS.
 (define (cf-path-prog sym prog-or-progs
                       :key (value #f) (default #f) (paths #f) (filter #f))
-  (unless (cf-have-subst? sym)
+  (unless (and (cf-have-subst? sym)
+               (not (string-null? (cf-ref sym))))
     (cf-msg-checking "for ~a" prog-or-progs)
     (if-let1 found (check-for-program (listify prog-or-progs)
                                       :paths paths :filter filter)
@@ -1193,7 +1253,7 @@
 ;; Feature Test API
 ;; Like AC_CHECK_HEADER.
 ;; Returns #t on success, #f on failure.
-(define (cf-check-header header-file :key (includes #f))
+(define (cf-header-available? header-file :key (includes #f))
   (let1 includes (or includes (cf-includes-default))
     (cf-msg-checking "~a usability" header-file)
     (rlet1 result (cf-try-compile (list includes
@@ -1201,6 +1261,7 @@
                                         #"#include <~|header-file|>\n")
                                   "")
       (cf-msg-result (if result "yes" "no")))))
+(define cf-check-header cf-header-available?) ;; autoconf compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_HEADERS.  Besides the check, it defines HAVE_<header-file>
@@ -1218,7 +1279,7 @@
 ;; Returns #t on success, #f on failure.
 ;; If TYPE is a valid type, sizeof(TYPE) compiles and sizeof((TYPE)) fails.
 ;; The second test is needed in case TYPE happens to be a variable.
-(define (cf-check-type type :key (includes #f))
+(define (cf-type-available? type :key (includes #f))
   (let1 includes (or includes (cf-includes-default))
     (cf-msg-checking "for ~a" type)
     (rlet1 result
@@ -1227,6 +1288,7 @@
              (not (cf-try-compile (list includes)
                                   #"if (sizeof ((~|type|))) return 0;")))
       (cf-msg-result (if result "yes" "no")))))
+(define cf-check-type cf-type-available?)  ; autoconf-compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_TYPES.
@@ -1244,7 +1306,7 @@
 ;; Like AC_CHECK_DECL
 ;; Returns #t on success, #f on failure.
 ;; Check SYMBOL is declared as a macro, a constant, a variable or a function.
-(define (cf-check-decl symbol :key (includes #f))
+(define (cf-decl-available? symbol :key (includes #f))
   (let1 includes (or includes (cf-includes-default))
     (cf-msg-checking "whether ~a is declared" symbol)
     (rlet1 result
@@ -1254,6 +1316,7 @@
                               #"#endif\n"
                               "return 0;"))
       (cf-msg-result (if result "yes" "no")))))
+(define cf-check-decl cf-decl-available?)  ;autoconf-compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_DECLS
@@ -1273,7 +1336,7 @@
 ;; Feature Test API
 ;; Like AC_CHECK_MEMBER
 ;; Works as a predicate
-(define (cf-check-member aggregate.member :key (includes #f))
+(define (cf-member-available? aggregate.member :key (includes #f))
   (receive (aggr memb) (string-scan aggregate.member #\. 'both)
     (unless (and aggr memb)
       (error "cf-check-member: argument doesn't contain a dot:"
@@ -1288,6 +1351,7 @@
                               (list #"static ~aggr ac_aggr;\n"
                                     #"if (sizeof ac_aggr.~|memb|) return 0;")))
         (cf-msg-result (if result "yes" "no"))))))
+(define cf-check-member cf-member-available?) ;autoconf-compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_MEMBERS
@@ -1304,7 +1368,7 @@
 ;; Like AC_CHECK_FUNC
 ;; NB: autoconf has language-dependent methods (AC_LANG_FUNC_LINK_TRY)
 ;; For now, we hardcode C.
-(define (cf-check-func func)
+(define (cf-func-available? func)
   (let1 includes (cf-includes-default)
     (cf-msg-checking #"for ~func")
     (rlet1 result ($ cf-try-compile-and-link
@@ -1321,10 +1385,11 @@
                        ,#"char ~func ();\n")
                      `(,#"return ~func ();"))
       (cf-msg-result (if result "yes" "no")))))
+(define cf-check-func cf-func-available?)  ;autoconf-compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_FUNCS
-(define (cf-check-funcs funcs :key (if-found identity) 
+(define (cf-check-funcs funcs :key (if-found identity)
                                    (if-not-found identity))
   (dolist [f funcs]
     (if (cf-check-func f)
@@ -1343,10 +1408,10 @@
 
 ;; Feature Test API
 ;; Like AC_CHECK_LIB
-(define (cf-check-lib lib fn
-                      :key (other-libs '()) 
-                           (if-found default-lib-found)
-                           (if-not-found default-lib-not-found))
+(define (cf-lib-available? lib fn
+                        :key (other-libs '())
+                        (if-found default-lib-found)
+                        (if-not-found default-lib-not-found))
   (let1 includes (cf-includes-default)
     (cf-msg-checking "for ~a in -l~a" fn lib)
     (if (with-cf-subst
@@ -1359,22 +1424,23 @@
       (begin
         (cf-msg-result "no")
         (if-not-found lib)))))
+(define cf-check-lib cf-lib-available?)    ;autoconf-compatible name
 
 (define (default-lib-search-found libname)
   (when libname
     (cf-subst-prepend 'LIBS #"-l~|libname|"))
   #t)
-   
+
 ;; Feature test API
 ;; Like AC_CHECK_LIBS
 (define (cf-search-libs fn libs
                         :key (other-libs '())
-                             (if-found default-lib-search-found) 
+                             (if-found default-lib-search-found)
                              (if-not-found default-lib-not-found))
   (let ([includes (cf-includes-default)]
         [xlibs #"~(string-join other-libs \" \") ~(cf$'LIBS)"])
     (define (try lib)
-      (with-cf-subst 
+      (with-cf-subst
        ([LIBS (if (eq? lib 'none) xlibs #"-l~|lib| ~xlibs")])
        (cf-try-compile-and-link includes
                                 (format "extern void ~a(); ~a();"
