@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.interactive - useful stuff in the interactive session
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,12 +35,15 @@
 
 (define-module gauche.interactive
   (export apropos d read-eval-print-loop print-mode
+          toplevel-reader-state
           ;; autoloaded symbols follow
           info info-page info-search reload ed
           reload-modified-modules module-reload-rules reload-verbose)
   )
 (select-module gauche.interactive)
 
+(autoload gauche.uvector f64vector uvector-alias u64vector-ref)
+
 ;;;
 ;;; Apropos - search bound symbols matching given pattern
 ;;;
@@ -68,9 +71,7 @@
         [matcher (cond [(symbol? item)
                         (let1 substr (symbol->string item)
                           (^[name] (string-scan name substr)))]
-                       [(string? item)
-                        ;; Note: future extension
-                        (error "Bad object for item: " item)]
+                       [(string? item) (^[name] (string-scan name item))]
                        [(is-a? item <regexp>) (^[name] (rxmatch item name))]
                        [else
                         (error "Bad object for item: " item)])]
@@ -86,10 +87,14 @@
            (when (matcher (symbol->string symbol))
              (found mod symbol))))))
 
+    (define (visible? sym)
+      (global-variable-bound? module sym))
+
     (define (found module symbol)
-      (set! result
-            (cons (format #f "~30s (~a)~%" symbol (module-name module))
-                  result)))
+      (push! result
+             (format #f "~30s (~a~a)~%" symbol
+                     (if (visible? symbol) "" "*")
+                     (module-name module))))
 
     ;; mimics the Scm_FindBinding
     (if stay-in-module
@@ -152,6 +157,29 @@
     (format #t ")\n")
     (values)))
 
+(define-method describe ((r <rational>))
+  (describe-common r)
+  (when (ratnum? r)
+    (format #t "  inexact: ~s\n" (inexact r)))
+  (values))
+
+(define-method describe ((d <real>))
+  (describe-common d)
+  (when (flonum? d)
+    (let* ([components (decode-float d)]
+           [buf (f64vector d)])
+      (if (finite? d)
+        (format #t "  mantissa: #x~a~14,'0,'_,4:x   exponent: ~d"
+                (if (positive? (vector-ref components 2)) "+" "-")
+                (vector-ref components 0)
+                (vector-ref components 1))
+        (format #t "  ~s" d))
+      (format #t "    hex: #x~16,'0,'_,4:x\n"
+              (u64vector-ref (uvector-alias <u64vector> buf) 0)))
+    (when (finite? d)
+      (format #t "  exact: ~s\n" (exact d))))
+  (values))
+
 (define-method describe ((g <generic>))
   (next-method) ; common object description
   (print "methods:")
@@ -168,11 +196,19 @@
 
 (define-method describe ((p <procedure>))
   (describe-common p)
-  (if-let1 source (source-location p)
+  (and-let1 source (source-location p)
     (format #t "Defined at ~s:~d\n" (car source) (cadr source)))
+  (format #t "type: ~s\n" (procedure-type p))
   (describe-slots p)
   (values))
 
+(define-method describe ((m <macro>))
+  (describe-common m)
+  (and-let1 source (assq-ref (~ m'info-alist) 'source-info)
+    (format #t "Defined at ~s:~d\n" (car source) (cadr source)))
+  (describe-slots m)
+  (values))
+
 (define d describe)
 
 ;;;
@@ -225,6 +261,19 @@
           (format "~a~a " *repl-name* delim)
           (format "~a[~a]~a " *repl-name* (module-name m) delim))))))
 
+
+;; can be
+;;  #f       - read edit isn't available at all
+;;  editable - read edit is being used
+;;  vanilla  - read edit is available, but turned off temporarily
+(define *read-edit-state* #f)
+
+(define toplevel-reader-state
+  (case-lambda
+    [() *read-edit-state*]
+    [(mode) (ecase mode
+              [(#f editable vanilla) (set! *read-edit-state* mode)])]))
+
 ;; Returns a reader procedure that can handle toplevel command.
 ;; READ - reads one sexpr from the REPL input
 ;; READ-LINE - read to the EOL from REPL input and returns a string.
@@ -243,24 +292,42 @@
           (unless (eof-object? expr) (skipper))
           expr)))))
 
-;; EXPERIMENTAL: Environment GAUCHE_READ_EDIT enables editing mode.
-;; Note that, at this moment, text.line-edit isn't complete; it doesn't
-;; handle multibyte characters nor the multiline expressions bigger
-;; than the screen height.  Once we complete text.line-edit, we make
-;; the feature available through command-line options of gosh.
+;; History file, used by input editor.
+;; The default history file is ~/.gosh_history.  The user can override
+;; it with the environment variable GAUCHE_HISTORY_FILE, unless
+;; the process is suid/sgid-ed.
+;; To prohibit history saving, set an empty string to "GAUCHE_HISTORY_FILE".
+(define (get-history-filename)
+  (let1 h (sys-getenv "GAUCHE_HISTORY_FILE")
+    (cond [(or (equal? h "") (sys-setugid?)) #f] ; do not save history file
+          [h    (sys-normalize-pathname h :absolute #t :expand #t)]
+          [else (sys-normalize-pathname "~/.gosh_history" :expand #t)])))
 
+;; The variable *read-edit* is #t by default, #f if env var
+;; GAUCHE_NO_READ_EDIT is set.  It is also controlled by -fread-edit or
+;; -fno-read-edit flag.
 (define-values (%prompter %reader %line-edit-ctx)
   (receive (r rl skipper ctx)
-      (if (sys-getenv "GAUCHE_READ_EDIT")
-        (make-editable-reader (^[] (default-prompt-string "$")))
+      (if (with-module gauche.internal *read-edit*)
+        (make-editable-reader (^[] (default-prompt-string "$"))
+                              (get-history-filename))
         (values #f #f #f #f))
+    (define vanilla-reader
+      (make-repl-reader read read-line consume-trailing-whitespaces))
     (if (and r rl skipper ctx)
-      (values (^[] #f)
-              (make-repl-reader r rl skipper)
-              ctx)
+      (let1 editing-reader (make-repl-reader r rl skipper)
+        (set! *read-edit-state* 'editable)
+        (values (^[]
+                  ;; if input editor is running, prompt is shown by the reader
+                  (when (eq? *read-edit-state* 'vanilla)
+                    (display (default-prompt-string)) (flush)))
+                (^[]
+                  (if (eq? *read-edit-state* 'editable)
+                    (editing-reader)
+                    (vanilla-reader)))
+                ctx))
       (values (^[] (display (default-prompt-string)) (flush))
-              (make-repl-reader read read-line
-                                consume-trailing-whitespaces)
+              vanilla-reader
               #f))))
 
 ;; error printing will be handled by the original read-eval-print-loop
@@ -313,22 +380,26 @@
 ;;;
 
 ;; EXPERIMENTAL: windows console code page support for text.line-edit
-;; NB: ces (character encoding scheme) conversion is not implemented.
 (cond-expand
  [gauche.os.windows
-  (autoload os.windows sys-get-console-output-cp)
-  ;; check if we have a windows console.
-  (when (or (sys-isatty (standard-input-port))
-            (sys-isatty (standard-output-port))
-            (sys-isatty (standard-error-port)))
-    ;; wide character settings for text.line-edit
-    (if-let1 ctx %line-edit-ctx
+  (autoload os.windows
+            sys-get-console-output-cp
+            sys-has-windows-console?
+            sys-windows-terminal?
+            sys-windows-console-legacy?)
+  ;; wide character settings for text.line-edit
+  (if-let1 ctx %line-edit-ctx
+    ;; check if we have a windows console
+    ;; (except for windows terminal (windows 10))
+    (when (and (sys-has-windows-console?)
+               (not (sys-windows-terminal?)))
       (case (sys-get-console-output-cp)
         [(65001)
          (set! (~ ctx 'wide-char-disp-setting 'mode) 'Surrogate)
          (set! (~ ctx 'wide-char-pos-setting  'mode) 'Surrogate)
          (set! (~ ctx 'wide-char-disp-setting 'wide-char-width) 2)
-         (set! (~ ctx 'wide-char-pos-setting  'wide-char-width) 1)
+         (set! (~ ctx 'wide-char-pos-setting  'wide-char-width)
+               (if (sys-windows-console-legacy?) 1 2))
          (set! (~ ctx 'wide-char-disp-setting 'surrogate-char-width) 2)
          (set! (~ ctx 'wide-char-pos-setting  'surrogate-char-width) 2)]
         [else ; 932 etc.
@@ -348,7 +419,7 @@
                         module-reload-rules reload-verbose)
 
 ;; Autoload editor invoker
-(autoload gauche.interactive.ed ed ed-pick-file)
+(autoload text.external-editor ed ed-pick-file)
 
 ;; Autoload toplevel command handler
 (autoload gauche.interactive.toplevel handle-toplevel-command)
@@ -358,4 +429,3 @@
 
 ;; This might help first time users
 (define-in-module user help "Type ,help (comma and help) for help")
-
