@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive/info.scm - online helper
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,9 +32,10 @@
 ;;;
 
 (define-module gauche.interactive.info
-  (use srfi-1)
+  (use scheme.list)
   (use srfi-13)
   (use text.info)
+  (use text.pager)
   (use file.util)
   (use util.match)
   (use util.levenshtein)
@@ -48,27 +49,35 @@
 (define *info-file* "gauche-refe.info")
 
 (define-class <repl-info> () ;; a singleton
-  ((info  :init-keyword :info)   ;; <info>
+  ((info  :init-keyword :info)   ;; <info> or #f
    (index :init-keyword :index)  ;; hashtable name -> [(node-name line-no)]
    ))
 
 (define get-info
   (let1 repl-info
       (delay
-        (let ([info  (open-info-file (find-info-file))]
+        (let ([info  (and-let1 path (find-info-file)
+                       (open-info-file path))]
               [index (make-hash-table 'string=?)])
-          (dolist [index-page '("Function and Syntax Index"
-                                "Module Index"
-                                "Variable Index")]
-            (dolist [p ($ info-parse-menu $ info-get-node info index-page)]
-              (hash-table-push! index (entry-name (car p)) (cdr p))))
-          ;; class index doesn't have surrounding '<>', but we want to search
-          ;; with them.
-          (dolist [p ($ info-parse-menu $ info-get-node info "Class Index")]
-            (hash-table-push! index #"<~(car p)>" (cdr p)))
-          ($ hash-table-for-each index
-             ;; reverse v here so that earlier entry listed first
-             (^[k v] (hash-table-put! index k (squash-entries (reverse v)))))
+          (if info
+            (begin
+              (dolist [index-page '("Function and Syntax Index"
+                                    "Module Index"
+                                    "Variable Index")]
+                (dolist [p ($ info-parse-menu $ info-get-node info index-page)]
+                  (hash-table-push! index (entry-name (car p)) (cdr p))))
+              ;; class index doesn't have surrounding '<>', but we want to search
+              ;; with them.
+              (dolist [p ($ info-parse-menu $ info-get-node info "Class Index")]
+                (hash-table-push! index #"<~(car p)>" (cdr p)))
+              ($ hash-table-for-each index
+                 ;; reverse v here so that earlier entry listed first
+                 (^[k v]
+                   (hash-table-put! index k (squash-entries (reverse v))))))
+            (warn "Couldn't find info document in ~s. \
+                   Maybe it has't been installed. \
+                   Check your Gauche installation.\n"
+                  (get-info-paths)))
           (make <repl-info>
             :info info :index index)))
     (^[] (force repl-info))))
@@ -87,57 +96,6 @@
 ;; " <n>" in the index entry.  This strips that.
 (define (entry-name e)
   (if-let1 m (#/ <\d+>$/ e) (rxmatch-before m) e))
-  
-(define *pager* (or (and-let1 s (sys-getenv "PAGER")
-                      (shell-tokenize-string s))
-                    (and-let1 cmd (or (find-file-in-paths "less")
-                                      (find-file-in-paths "more"))
-                      (list cmd))))
-
-(define (viewer-pager s)
-  (let1 p (run-process *pager* :input :pipe)
-    (guard (e (else #f))
-      (display s (process-input p)))
-    (close-output-port (process-input p))
-    (process-wait p)))
-
-(define (viewer-dumb s) (display s))
-
-(define (viewer-nounicode s)
-  (display ($ regexp-replace-all* s
-              #/\u21d2/ "==>"      ; @result{}
-              #/\u2026/ "..."      ; @dots{}
-              #/\u2018/ "`"        ; @code{}
-              #/\u2019/ "'"        ; @code{}
-              #/\u201C/ "``"       ; ``         (e.g. ,i do)
-              #/\u201D/ "''"       ; ''         (e.g. ,i do)
-              #/\u2261/ "=="       ; @equiv{}   (e.g. ,i cut)
-              #/\u2212/ "-"        ; @minus     (e.g. ,i modulo)
-              #/\u2022/ "*"        ; @bullet    (e.g. ,i lambda)
-              #/\u2013/ "--"       ; --         (e.g. ,i utf8-length)
-              #/\u2014/ "---"      ; ---        (e.g. ,i lambda)
-              #/\u00df/ "[Eszett]" ; eszett     (e.g. ,i char-upcase)
-              )))
-
-(define viewer
-  (cond [(cond-expand
-          [(and gauche.os.windows
-                (not gauche.ces.none))
-           (use os.windows)
-           (guard (e [else #f])
-             (and (sys-isatty 1)
-                  (sys-get-console-mode 
-                   (sys-get-std-handle STD_OUTPUT_HANDLE))
-                  (not (= (sys-get-console-output-cp) 65001))))] ;unicode cp
-          [else #f])
-         viewer-nounicode]
-        [(or (equal? (sys-getenv "TERM") "emacs")
-             (equal? (sys-getenv "TERM") "dumb")
-             (not (sys-isatty (current-output-port)))
-             (not *pager*))
-         viewer-dumb]
-        [else 
-         viewer-pager]))
 
 (define (get-info-paths)
   (let* ([syspath (cond [(sys-getenv "INFOPATH") => (cut string-split <> #\:)]
@@ -151,14 +109,11 @@
     (append in-place syspath instpath)))
 
 (define (find-info-file)
-  (let1 paths (get-info-paths)
-    (or (find-file-in-paths *info-file*
-                            :paths paths
-                            :pred (^p (or (file-is-readable? p)
-                                          (file-is-readable? #"~|p|.gz")
-                                          (file-is-readable? #"~|p|.bz2"))))
-        (errorf "couldn't find info file ~s in paths: ~s" *info-file* paths))
-    ))
+  (find-file-in-paths *info-file*
+                      :paths (get-info-paths)
+                      :pred (^p (or (file-is-readable? p)
+                                    (file-is-readable? #"~|p|.gz")
+                                    (file-is-readable? #"~|p|.bz2")))))
 
 (define (handle-ambiguous-name entry-name)
   (let* ([keys (map x->string (hash-table-keys (~ (get-info)'index)))]
@@ -201,20 +156,26 @@
   (values))
 
 ;; API
+;; NB: The info slot may be #f, but in that case key never hits, so
+;; the proc won't be called.
 (define (info key)
   ($ lookup&show key
      (^[node&line]
        (let1 node (info-get-node (~ (get-info)'info) (car node&line))
-         (viewer (if (null? (cdr node&line))
-                   (~ node'content)
-                   (info-extract-definition node (cadr node&line))))))))
+         (display/pager
+          (if (null? (cdr node&line))
+            (~ node'content)
+            (info-extract-definition node (cadr node&line))))))))
 
 ;; API
+;; NB: The info slot may be #f, but in that case key never hits, so
+;; the proc won't be called.
 (define (info-page key)
   ($ lookup&show key
      (^[node&line]
-       (viewer (~ (info-get-node (~ (get-info)'info) (car node&line))
-                  'content)))))
+       (display/pager
+        (~ (info-get-node (~ (get-info)'info) (car node&line))
+           'content)))))
 
 ;;
 ;; Search info entries by regexp
@@ -244,8 +205,7 @@
   (let1 entries (search-entries rx)
     (if (null? entries)
       (print #"No entry matching ~|rx|")
-      (viewer (with-output-to-string
-                (^[] (for-each format-search-result-entry entries))))))
+      (display/pager
+       (with-output-to-string
+         (^[] (for-each format-search-result-entry entries))))))
   (values))
-
-               
