@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive/toplevel.scm - toplevel commands
 ;;;
-;;;   Copyright (c) 2015-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2015-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -58,6 +58,8 @@
 (select-module gauche.interactive.toplevel)
 
 (autoload file.util home-directory expand-path)
+(autoload text.pager with-output-to-pager)
+(autoload text.line-edit default-keymap command-name->keystroke-string)
 
 ;; toplevel-commands
 ;; Map from symbol to (parser help-message proc)
@@ -218,22 +220,50 @@
         (cons cmd help)))
     (match args
       [()
-       (print "You're in REPL (read-eval-print-loop) of Gauche shell.")
-       (print "Type a Scheme expression to evaluate.")
-       (print "Evaluate (exit) to exit REPL.")
-       (print "A word preceded with comma has special meaning.  Type ,help <cmd> ")
-       (print "to see the detailed help for <cmd>.")
-       (print "Commands can be abbreviated as far as it is not ambiguous.")
-       (print)
-       (dolist [cmd&help
-                (sort (map (^p (get-cmd&help (cdr p)))
-                           (toplevel-command-keys))
-                      string<? car)]
-         (format #t (if (> (string-length (car cmd&help)) 10)
-                      " ,~10a\n             ~a\n"
-                      " ,~10a ~a\n")
-                 (car cmd&help)
-                 (cdr cmd&help)))
+       (with-output-to-pager
+        (^[]
+          (print "You're in REPL (read-eval-print-loop) of Gauche shell.")
+          (print "Type a Scheme expression to evaluate.")
+          (print "Evaluate (exit) to exit REPL.")
+          (print "A word preceded with comma has special meaning.  Type ,help <cmd> ")
+          (print "to see the detailed help for <cmd>.")
+          (print "Commands can be abbreviated as far as it is not ambiguous.")
+          (print)
+          (dolist [cmd&help
+                   (sort (map (^p (get-cmd&help (cdr p)))
+                              (toplevel-command-keys))
+                         string<? car)]
+            (format #t (if (> (string-length (car cmd&help)) 10)
+                         " ,~10a\n             ~a\n"
+                         " ,~10a ~a\n")
+                    (car cmd&help)
+                    (cdr cmd&help)))
+          (case (toplevel-reader-state)
+            [(editable)
+             (let ()
+               (define (lookup name)
+                 (command-name->keystroke-string (default-keymap) name))
+               (define c-f (lookup 'forward-char))
+               (define c-b (lookup 'backward-char))
+               (define c-p (lookup 'prev-line-or-history))
+               (define c-n (lookup 'next-line-or-history))
+               (define hmap (lookup 'help-binding-command))
+               (define hkey (lookup 'help-keystroke-command))
+               (print)
+               (print "You can also edit input using Emacs-like key bindings.")
+               (print "Use "c-f", "c-b", "c-p" and "c-n" for foward, backward,")
+               (print "previous line or next line.  If you're the beginning of")
+               (print "the input expression and you type "c-p", the previous expression")
+               (print "is shown if there's any.  "c-n" is similar.")
+               (print "Type "hmap" to see all the key bindings, and "hkey" <key>")
+               (print "to see the description of a particular <key>.")
+               (print)
+               (print "If you have a trouble with input line editor, type \",edit off\"")
+               (print "to turn off input editing."))]
+            [(vanilla)
+             (print)
+             (print "You can also turn on line-editing mode by \",edit on\".")]
+            )))
        *no-value*]
       [(('unquote cmd)) ((toplevel-command-helper cmd)) *no-value*]
       [(cmd) ((toplevel-command-helper cmd)) *no-value*]
@@ -260,6 +290,31 @@
         (begin (sys-chdir dir) (sys-getcwd))
         (usage)))))
 
+;; Turn on/off input editing.
+(define-toplevel-command edit :read
+  " [on|off]\
+ \nQuery, or turn on/off input editing mode.\
+ \nWithout arguments, show the current input editing mode.  With on or off, turns\
+ \non or off input editing mode.  Note that if gosh hasn't been started with input\
+ \nediting feature, you can't turn it on afterwards."
+  (^[args]
+    (match args
+      [() (case (toplevel-reader-state)
+            [(#f)
+             (print "Input editing is not available in the current session.")]
+            [(editable)
+             (print "Input editing is enabled.  Type ',edit off' to turn it off.")]
+            [(vanilla)
+             (print "Input editing is temporarily disabled.  Type ',edit on' to turn it on.")])]
+      [('on)  (if (toplevel-reader-state)
+                (toplevel-reader-state 'editable)
+                (error "You can't turn on input editor in this session."))]
+      [('off) (if (toplevel-reader-state)
+                (toplevel-reader-state 'vanilla)
+               (error "You're not using input editor in this session."))]
+      [_ (usage)])
+    *no-value*))
+
 ;; Run shell command.
 ;; A tradition to use '!' for shell escape, but "comma - exclamation-mark"
 ;; combination is a bit awkward to type.  "comma - s - h" is much easier.
@@ -375,7 +430,11 @@
  \nbuffer, for the copy&paste, etc."
   (^[exprs]
     (match exprs
-      [() (write *1 (make-write-controls)) (newline)]
+      ;; TODO: Get console width if possible
+      [()
+       (write *1 (write-controls-copy (print-mode)
+                                      :pretty #t :width 79
+                                      :length #f :level #f))
+       (newline)]
       [_ (usage)])
     *no-value*))
-
