@@ -1,7 +1,7 @@
 ;;;
 ;;; instance-pool.scm - instance pool metaclass
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,10 +31,8 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; EXPERIMENTAL.   THE API MAY CHANGE.
-
 (define-module gauche.mop.instance-pool
-  (use srfi-1)
+  (use scheme.list)
   (export <instance-pool-meta> <instance-pool-mixin>
           <instance-table-meta> <instance-table-mixin>
           instance-pool-of instance-pools-of
@@ -180,5 +178,3 @@
   (instance-pool-for-each (class-of self) proc))
 (define-method instance-pool-map ((self <instance-pool-mixin>) proc)
   (instance-pool-map (class-of self) proc))
-
-
