@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.test - test framework
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -83,15 +83,22 @@
 
 (define-module gauche.test
   (export test test* test-start test-end test-running? test-section test-log
-          test-module test-script
+          test-module test-script test*/diff
           test-error test-one-of test-none-of
-          test-check test-record-file test-summary-check
+          test-check test-check-diff
+          test-report-failure test-report-failure-plain
+          test-report-failure-diff
+          test-record-file test-summary-check
           *test-error* *test-report-error* test-error? prim-test
-          test-count++ test-pass++ test-fail++))
+          test-count++ test-pass++ test-fail++
+
+          test-remove-files test-with-temporary-directory))
 (select-module gauche.test)
 
-;; Autoload test-script to avoid depending other modules
+;; Autoloads to avoid depending other modules
 (autoload "gauche/test/script" test-script)
+(autoload "gauche/test/diff"
+          (:macro test*/diff) test-check-diff test-report-failure-diff)
 
 ;; An object to represent error.  This class isn't exported; the user
 ;; must use `test-error' procedure to create an instance.
@@ -180,9 +187,10 @@
   (vector-set! *test-counts* 0 (+ (vector-ref *test-counts* 0) 1)))
 (define (test-pass++)
   (vector-set! *test-counts* 1 (+ (vector-ref *test-counts* 1) 1)))
-(define (test-fail++ msg expected result)
+(define (test-fail++ msg expected result :optional (report test-report-failure))
   (vector-set! *test-counts* 2 (+ (vector-ref *test-counts* 2) 1))
-  (set! *discrepancy-list* (cons (list msg expected result) *discrepancy-list*)))
+  (set! *discrepancy-list*
+        (cons (list report msg expected result) *discrepancy-list*)))
 (define (format-summary)
   (format "Total: ~5d tests, ~5d passed, ~5d failed, ~5d aborted.\n"
           (vector-ref *test-counts* 0)
@@ -218,24 +226,52 @@
 
 ;; Tests ------------------------------------------------------------
 
+;; test msg expect thunk :optional check report hook
+;;   check is called (check expected actual-result).  default is test-check.
+;;   report is called (report msg expected actual-result) when failed.
+;;                  The output must end with newline.
+;;   hook is called (hook 'fail|'pass msg expected actual-result)
+;; NB: In 0.9.10, we have a signature ':optional check hook' (no report).
+;; We adopted ':optional check report hook', for it makes more sense.
+;; To keep the backward compatibility, we check the arity of the second
+;; optional argument.
+
 ;; Primitive test.  This uses neither with-error-handler nor the
 ;; object system, so it can be used _before_ those constructs are tested.
-(define (prim-test msg expect thunk . compare)
-  (let ([cmp (if (pair? compare) (car compare) test-check)])
+(define (prim-test msg expect thunk . args)
+  (let ([cmp  (or (and (pair? args) (car args))
+                  test-check)]
+        [report (or (and (pair? args) (pair? (cdr args))
+                         (cadr args))
+                    test-report-failure)]
+        [hook (and (pair? args) (pair? (cdr args)) (pair? (cddr args))
+                   (caddr args))])
+    ;; TRANSIENT: Backward compatibility of 'hook'.  Remove by 1.0.
+    (when (and (not (eq? report test-report-failure))
+               (not hook)
+               (eqv? (arity report) 4))
+      (warn "gauche.test: `test' is called with old signature (hook).\n")
+      (set! hook report)
+      (set! report test-report-failure))
+    ;; End transient code
     (format/ss #t "test ~a, expects ~s ==> " msg expect)
     (flush)
     (test-count++)
     (let ([r (thunk)])
       (cond [(cmp expect r)
              (format #t "ok\n")
-             (test-pass++)]
+             (test-pass++)
+             (when hook (hook 'pass msg expect r))]
             [else
-             (format/ss #t "ERROR: GOT ~S\n" r)
-             (test-fail++ msg expect r)])
+             (display "ERROR: GOT ")
+             (report msg expect r)
+             (newline)
+             (test-fail++ msg expect r report)
+             (when hook (hook 'fail msg expect r))])
       (flush))))
 
 ;; Normal test.
-(define (test msg expect thunk . compare)
+(define (test msg expect thunk . args)
   (apply prim-test msg expect
          (lambda ()
            (guard (e [else
@@ -244,20 +280,18 @@
                       (make <test-error> :condition e
                             :class (class-of e)
                             :message
-                            (if (condition-has-type? e <message-condition>)
-                              (slot-ref e 'message)
-                              e))])
+                            (condition-message e e))])
              (thunk)))
-         compare))
+         args))
 
 ;; A convenient macro version
-;; We use er-macro-transformer, so test* should be used after macro
+;; We use er-macro-transformer, so test* should be used after the macro
 ;; subsystem is tested with more primitive framework.
 (define-syntax test*
   (er-macro-transformer
    (lambda (f r c)
-     (apply (lambda (_ msg expect form . compare)
-              `(,(r 'test) ,msg ,expect (,(r 'lambda) () ,form) ,@compare))
+     (apply (lambda (_ msg expect form . args)
+              `(,(r 'test) ,msg ,expect (,(r 'lambda) () ,form) ,@args))
             f))))
 
 ;; Toplevel binding sanity check ----------------------------------
@@ -398,7 +432,7 @@
           [(and (pair? (car code))
                 (memq (caar code) gref-call-insns))
            (if (pair? (cdr code))
-             (if (identifier? (cadr code))
+             (if (wrapped-identifier? (cadr code))
                (let* ([src-code (assq-ref (assv-ref debug-info i '())
                                           'source-info)]
                       ;; If the identifier is in `code' and the source-code
@@ -421,7 +455,7 @@
                        debug-info))
                (loop r (cddr code) (+ i 2) debug-info))    ;skip #<gloc>
              (loop r '() (+ i 1) debug-info))]
-          [(identifier? (car code))
+          [(wrapped-identifier? (car code))
            (loop `((,(car code) #f #f) ,@r) (cdr code) (+ i 1) debug-info)]
           [(is-a? (car code) <compiled-code>)
            (loop (loop r (code->list (car code)) 0 (~ (car code)'debug-info))
@@ -455,6 +489,14 @@
 
 ;; Logging and bookkeeping -----------------------------------------
 
+;; API: Default report procedure
+(define (test-report-failure msg expected actual)
+  (write actual))
+
+;; API: Use ~a instead of ~s
+(define (test-report-failure-plain msg expected actual)
+  (display actual))
+
 ;; private global flag, true during we're running tests.
 ;; (we avoid using parameters intentionally.)
 (define *test-running* #f)
@@ -521,7 +563,11 @@
       (begin
         (fmt "failed.\ndiscrepancies found.  Errors are:\n")
         (for-each (lambda (r)
-                    (apply fmt "test ~a: expects ~s => got ~s\n" r))
+                    (apply (lambda (report msg expect actual)
+                             (fmt "test ~a: expects ~s => got ~a\n" msg expect
+                                  (with-output-to-string
+                                    (cut report msg expect actual))))
+                           r))
                   (reverse *discrepancy-list*))))
     (flush)
 
@@ -544,3 +590,28 @@
   (unless (and (zero? (vector-ref *test-counts* 2))
                (zero? (vector-ref *test-counts* 3)))
     (exit 1)))
+
+;; Temporary files and cleanup  --------------------------------------
+
+;; Remove files and directories.  Functionally same as 'remove-files' in
+;; file.util; but this can be used _before_ we test file.util.
+(define (test-remove-files . paths)
+  (define (remove-1 path)
+    (if (and (file-is-directory? path)
+             (not (eq? (slot-ref (sys-lstat path)'type) 'symlink)))
+      (begin (for-each (^e (unless (member e '("." ".."))
+                             (remove-1 (string-append path "/" e))))
+                       (sys-readdir path))
+             (sys-rmdir path))
+      (sys-unlink path)))
+  (for-each remove-1 paths))
+
+;; Create a fresh DIR, call THUNK (without cd'ing), then remove DIR.
+(define (test-with-temporary-directory dir thunk)
+  (test-remove-files dir)
+  (sys-mkdir dir #o755)
+  ;; Avoid relying on unwind-protect
+  (dynamic-wind
+    (^[] #f)
+    thunk
+    (^[] (test-remove-files dir))))
