@@ -1,7 +1,7 @@
 ;;;
 ;;; rfc.ip - Internet Protocol
 ;;;
-;;;   Copyright (c) 2007-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2007-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,7 +37,7 @@
   (use gauche.uvector)
   (use binary.io)
   (use util.match)
-  (use srfi-1)
+  (use scheme.list)
   (use srfi-13)
   (export ip-version ip-header-length ip-protocol
           ip-source-address ip-destination-address
@@ -117,4 +117,3 @@
              (memv x/24 '(#xc00002    ; 192.0.2.0/24 test-net
                           #xc05863))) ; 192.88.99.0/24 6to4 anycast relay
          )))
-
