@@ -1,7 +1,7 @@
 ;;;
 ;;; html-lite.scm - lightweight HTML construction
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 (define-module text.html-lite
   (use text.tree)
-  (use srfi-1)
+  (use scheme.list)
   (export html-escape html-escape-string html-doctype)
   )
 (select-module text.html-lite)
