@@ -1,7 +1,7 @@
 ;;;
 ;;; info.scm - parse info file
 ;;;
-;;;   Copyright (c) 2000-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,12 +33,12 @@
 
 (define-module text.info
   (use srfi-13)
-  (use srfi-14)
+  (use scheme.charset)
   (use text.parse)
   (use gauche.process)
   (use file.util)
   (cond-expand
-   [gauche.sys.zlib 
+   [gauche.sys.zlib
     (use rfc.zlib)]
    [else])
   (export <info-file> <info-node>
@@ -185,7 +185,7 @@
 ;; Menu entry:
 ;;    (<entry-name> <node-name> [<line-number>])
 ;; Where <entry-name> is either a node name, function or macro name,
-;; module name, 
+;; module name,
 (define-method info-parse-menu ((info <info-node>))
   (with-input-from-string (~ info 'content)
     (^[]
@@ -267,4 +267,3 @@
                        [(#/^$/ line) (print) (desc (read-line))]
                        [(#/^ {4}/ line) (print line) (desc (read-line))]
                        [else]))])))))
-
