@@ -1,7 +1,7 @@
 ;;;
 ;;; parse.scm - utilities to parse input
 ;;;
-;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,7 +44,7 @@
 
 (define-module text.parse
   (use srfi-13)
-  (use srfi-14)
+  (use scheme.charset)
   (use util.match)
   (export find-string-from-port?
           assert-curr-char
